// Parametric model of WindFloat design for GeniE
// Written by Vetle Kallaak 2020


// From opening new workspace
GenieRules.Compatibility.version = "V7.13-11";
GenieRules.BeamCreation.DefaultCurveOffset = ReparameterizedBeamCurveOffset();
GenieRules.Transformation.DefaultConnectedCopy = false;
GenieRules.Units.setInputUnit(Angle, "deg");
// Tolerances Rules
GenieRules.Tolerances.useTolerantModelling = true;
GenieRules.Tolerances.pointTolerance = 0.01 m;
GenieRules.Tolerances.angleTolerance = 2 deg;


// Parametric dimensions
// MATLAB WRITES LINE 18-24 and 27-29!!
numCol = 3; // Number of columns
draft = 22.3m; // Floater draft
diaCol = 11.5m; // Column diameter
FB1 = 1m; // Modeled freebord
dcc = 68.5m; // Distance center-center columns
lhp = 17.5m; // Heave plate length of plate edge (hexagon)
hhpM = 0.4m; // Modeled height of heave plate

// For mesh
numElWL2 = 26; // Elemenents on WL column 2
Mesh_Length_P = 1.4m; // General mesh size
Mesh_max = 2.2m; // Max mesh size, for under hp

// GeniE commands
numElWL1 = numElWL2/4; // Elements along water line of column 1
chp = (diaCol/2+lhp*Math.cos(30*Math.PI/180))/2; // Center of heave plate

Mesh_Length_chptop = (Mesh_Length_P + hhpM)/2; // Mesh size at center hp top side
Mesh_Length_chpbot = (Mesh_max + hhpM)/2; // Mesh size at center hp bottom side
NumElWL1 = NumberOfElements(numElWL1); 
NumElWL2 = NumberOfElements(numElWL2); // Should be 4 times NumElWL1
NumELhps = NumberOfElements(1); // Number of elements hhp

denshp_topc = MeshDensity(Mesh_Length_chptop);
denshp_botc = MeshDensity(Mesh_Length_chpbot);
denshp_botinner = MeshDensity(Mesh_max);

// Main mesh size
Md_def = MeshDensity(Mesh_Length_P);// Represent the maximum element size. Create a mesh density that can be applied to a plate, a beam or a feature edge.
Md_def.growthRate = 1.1;
Md_def.setDefault();

// Mesh rules from Erin Bachynski
//  General
GenieRules.Meshing.elementType = mp1stOrder; // 1. order elements
GenieRules.Meshing.superElementType = 1; // Edit if creating diff versions of models, 1 for panel model
GenieRules.Meshing.autoSimplifyTopology = false; // Removes superfluous topology lines/curves, better mesh
GenieRules.Meshing.autoSplitPeriodicGeometry = true; // Help with mesh
GenieRules.Meshing.preference(mpPreferRectangularMesh, false); // Use with care
GenieRules.Meshing.preference(mpAllowTriangularElements, false); // Disable to use as few as possible
GenieRules.Meshing.preference(mpPreferPointMassAsNodeMass, true);
GenieRules.Meshing.preference(mpUseDrillingElements, false);
GenieRules.Meshing.preference(mpUseEccentricHinges, true);

GenieRules.Meshing.scantlings = msGross;
GenieRules.Meshing.ignoreEccentricities = false;
GenieRules.Meshing.useCocentricBeams = false;

GenieRules.Meshing.preference(mpIncludeUnusedProperties, false);
GenieRules.Meshing.preference(mpMeshDensityRounded, true);
GenieRules.Meshing.preference(mpAdjustNumberOfElements, true); // Should be checked
GenieRules.Meshing.preference(mpWriteLoadsSeparate, true); // Should be checked

GenieRules.Meshing.preference(mpUseLongLoadcaseNames, false);
GenieRules.Meshing.preference(mpUseLongSetNames, false);
GenieRules.Meshing.preference(mpUseLongPropertyNames, false);

//GenieRules.Meshing.faceMeshStrategy = TemplateQuadMesher;
//GenieRules.Meshing.faceMeshStrategy = SesamQuadMesher;
GenieRules.Meshing.edgeMeshStrategy = LinearDistributionEdge;

GenieRules.Meshing.eliminateInternalVertices = true; // Better mesh
GenieRules.Meshing.eliminateInternalEdges = false; // False for allowing feature edges
GenieRules.Meshing.useUniformizedFaceParameterization = false; 
//  Max/min angle
GenieRules.Meshing.activate(mpMaxAngle, mpFail, true);
GenieRules.Meshing.setLimit(mpMaxAngle, mpFail, 179 deg);
GenieRules.Meshing.activate(mpMaxAngle, mpSplit, false);
GenieRules.Meshing.setLimit(mpMaxAngle, mpSplit, 165 deg); //
GenieRules.Meshing.activate(mpMinAngle, mpFail, true); // 
GenieRules.Meshing.setLimit(mpMinAngle, mpFail, 1 deg);
GenieRules.Meshing.activate(mpMinAngle, mpSplit, false);
GenieRules.Meshing.setLimit(mpMinAngle, mpSplit, 15 deg); // 
//  Jacobi 
GenieRules.Meshing.activate(mpMaxRelativeJacobi, mpFail, false);
GenieRules.Meshing.setLimit(mpMaxRelativeJacobi, mpFail, 10);
GenieRules.Meshing.activate(mpMaxRelativeJacobi, mpSplit, false);
GenieRules.Meshing.setLimit(mpMaxRelativeJacobi, mpSplit, 5);
GenieRules.Meshing.activate(mpMinNormalizedJacobi, mpFail, false);
GenieRules.Meshing.setLimit(mpMinNormalizedJacobi, mpFail, 0);
GenieRules.Meshing.activate(mpMinNormalizedJacobi, mpSplit, false);
GenieRules.Meshing.setLimit(mpMinNormalizedJacobi, mpSplit, 0.2);
//  Eliminate edge
GenieRules.Meshing.activate(mpMinEdge, false); 
GenieRules.Meshing.setLimit(mpMinEdge, 0.1);
//  Chord height
GenieRules.Meshing.activate(mpMaxChord, false); // Increase mesh at curved plate 
GenieRules.Meshing.setLimit(mpMaxChord, 0.2);

// Can't find these
GenieRules.Meshing.activate(mpMaxTwistAngle, mpFail, false);
GenieRules.Meshing.setLimit(mpMaxTwistAngle, mpFail, 30 deg);
GenieRules.Meshing.activate(mpMaxTwistAngle, mpSplit, false);
GenieRules.Meshing.setLimit(mpMaxTwistAngle, mpSplit, 10 deg);

// Define properties (if needed)
// Tck1 = Thickness(0);
// Tck1.setDefault();

// Modeling points

// Columns
P11 = Point(dcc/2/Math.cos(30*Math.PI/180),0.0,FB1);
P12 = Point(dcc/2/Math.cos(30*Math.PI/180),0.0,-draft+hhpM);
P13 = Point(dcc/2/Math.cos(30*Math.PI/180),0.0,-draft);

P21 = Point(-dcc/2*Math.tan(30*Math.PI/180),dcc/2,FB1);
P22 = Point(-dcc/2*Math.tan(30*Math.PI/180),dcc/2,-draft+hhpM);
P23 = Point(-dcc/2*Math.tan(30*Math.PI/180),dcc/2,-draft);

// Heave plates
var myPoints = new Array(18);
// Heave plate 2, entire
for (var i = 0; i < 6; i++) {
myPoints[i] = P22.copyTranslate(Vector3d(lhp*Math.cos(Math.PI/2+60*i*Math.PI/180),lhp*Math.sin(Math.PI/2+60*i*Math.PI/180),0));
myPoints[i+6] = P23.copyTranslate(Vector3d(lhp*Math.cos(Math.PI/2+60*i*Math.PI/180),lhp*Math.sin(Math.PI/2+60*i*Math.PI/180),0));
myPoints[i].name = "P_hp2top" + i;
myPoints[i+6].name = "P_hp2bot" + i;
}
// Heave plate 1, half
for (var i = -1; i <= 1; i++) {
myPoints[i+13] = P12.copyTranslate(Vector3d(lhp*Math.cos(Math.PI/2+60*i*Math.PI/180),lhp*Math.sin(Math.PI/2+60*i*Math.PI/180),0));
myPoints[i+16] = P13.copyTranslate(Vector3d(lhp*Math.cos(Math.PI/2+60*i*Math.PI/180),lhp*Math.sin(Math.PI/2+60*i*Math.PI/180),0));
myPoints[i+13].name = "P_hp1top" + (i+1);
myPoints[i+16].name = "P_hp1bot" + (i+1);	
}

P_hp1top3 = P12.copyTranslate(Vector3d(-lhp*Math.cos(30*Math.PI/180),0,0));
P_hp1bot3 = P13.copyTranslate(Vector3d(-lhp*Math.cos(30*Math.PI/180),0,0));
P_hp1top4 = P12.copyTranslate(Vector3d(lhp*Math.cos(30*Math.PI/180),0,0));
P_hp1bot4 = P13.copyTranslate(Vector3d(lhp*Math.cos(30*Math.PI/180),0,0));

// Modeling shells

// Shells of columns
Pl1 = CreateShellCircularConeCylinder(P11, diaCol/2, P12, diaCol/2, -180, 0);
Pl2 = CreateShellCircularConeCylinder(P21, diaCol/2, P22, diaCol/2, 0, 360);


// Shells of heave plate 1, entire 
Pl3 = Plate(P_hp2top0,P_hp2top1,P_hp2top2,P_hp2top3,P_hp2top4,P_hp2top5);
Validate(Pl3.primitivePartCount == 2);
Pl3.explode(IndexedNameMask(4));
Validate(Pl4);
Validate(Pl5);
Delete(Pl5);

Pl_hp2bot = Plate(P_hp2bot0,P_hp2bot1,P_hp2bot2,P_hp2bot3,P_hp2bot4,P_hp2bot5);
Pl_hp2bot.flipNormal();

var Plates = new Array(5);

for (var i = 0; i < 5; i++) {
Plates[i] = Plate(myPoints[i],myPoints[i+6],myPoints[i+7],myPoints[i+1]);
Plates[i].name = "Pl_hp2s" + i;
}
Pl_hp2s5 = Plate(P_hp2top5,P_hp2bot5,P_hp2bot0,P_hp2top0);


// Shells of heave plate 2, half

Pl3 = Plate(P_hp1top0,P_hp1top1,P_hp1top2,P_hp1top3,P_hp1top4);
Validate(Pl3.primitivePartCount == 2);
Pl3.explode(IndexedNameMask(10));
Validate(Pl10);
Validate(Pl11);
Delete(Pl10);

Pl_hp1bot = Plate(P_hp1bot0,P_hp1bot1,P_hp1bot2,P_hp1bot3,P_hp1bot4);
Pl_hp1bot.flipNormal();

Pl_hp1s0 = Plate(P_hp1top0,P_hp1bot0,P_hp1bot1,P_hp1top1);
Pl_hp1s1 = Plate(P_hp1top1,P_hp1bot1,P_hp1bot2,P_hp1top2);
Pl_hp1s2 = Plate(P_hp1top2,P_hp1bot2,P_hp1bot3,P_hp1top3);
Pl_hp1s3 = Plate(P_hp1top4,P_hp1bot4,P_hp1bot0,P_hp1top0);


// Make more points and lines, for controling mesh
P11_1 = P11.copyTranslate(Vector3d(-diaCol/2,0,0));
P11_2 = P11.copyTranslate(Vector3d(diaCol/2,0,0));
P11_3 = P11.copyTranslate(Vector3d(0,diaCol/2,0));
Curve_top11 = GuideArcElliptic(P11, P11_2, P11_3, true);
Curve_top12 = Curve_top11.copyRotate(P11,Vector3d(0,0,1),90*1);
Curve_WL11 = Curve_top11.copyTranslate(Vector3d(0,0,-FB1));
Curve_WL12 = Curve_top12.copyTranslate(Vector3d(0,0,-FB1));
Curve_low11 = Curve_top11.copyTranslate(Vector3d(0,0,-FB1-draft+hhpM));
Curve_low12 = Curve_top12.copyTranslate(Vector3d(0,0,-FB1-draft+hhpM));
Curve_bot11 = Curve_top11.copyTranslate(Vector3d(0,0,-FB1-draft));
Curve_bot12 = Curve_top12.copyTranslate(Vector3d(0,0,-FB1-draft));

Curve_top2 = CreateCircleFromPlaneAndRadius(P21,Vector3d(0,0,1),diaCol/2); 
Curve_WL2 = Curve_top2.CopyTranslate(Vector3d(0,0,-FB1));
Curve_low2 = Curve_top2.CopyTranslate(Vector3d(0,0,-FB1-draft+hhpM));
Curve_bot2 = Curve_top2.CopyTranslate(Vector3d(0,0,-FB1-draft));

// Top shell for columns
Pl2_top = CoverCurves(Curve_top2);
Pl2_top.flipNormal();
Pl1_top = CoverCurves(Curve_top11,Curve_top12);
Pl1_top.flipNormal();


// Define Feature Edges for mesh

// On columns and heave plates
FEdge_WL2 = FeatureEdge(Curve_WL2);
FEdge_WL2.numberOfElements = NumElWL2;

FEdge_top2 = FeatureEdge(Curve_top2);
FEdge_top2.numberOfElements = NumElWL2;

FEdge_low2 = FeatureEdge(Curve_low2);
FEdge_low2.numberOfElements = NumElWL2;

FEdge_bot2 = FeatureEdge(Curve_bot2);
FEdge_bot2.meshDensity = denshp_botinner;

FEdge_WL11 = FeatureEdge(Curve_WL11);
FEdge_WL11.numberOfElements = NumElWL1;
FEdge_WL12 = FeatureEdge(Curve_WL12);
FEdge_WL12.numberOfElements = NumElWL1;

FEdge_top11 = FeatureEdge(Curve_top11);
FEdge_top11.numberOfElements = NumElWL1;
FEdge_top12 = FeatureEdge(Curve_top12);
FEdge_top12.numberOfElements = NumElWL1;

FEdge_low11 = FeatureEdge(Curve_low11);
FEdge_low11.numberOfElements = NumElWL1;
FEdge_low12 = FeatureEdge(Curve_low12);
FEdge_low12.numberOfElements = NumElWL1;

FEdge_bot11 = FeatureEdge(Curve_bot11);
FEdge_bot11.meshDensity = denshp_botinner;
FEdge_bot12 = FeatureEdge(Curve_bot12);
FEdge_bot12.meshDensity = denshp_botinner;

// Num elements on sides of heave plates
var sideLines = new Array(11);
var sideFEdges = new Array(11);
for (var i =  0; i < 6; i++) {
sideLines[i]=GuideLine(myPoints[i],myPoints[i+6],0);
sideFEdges[i]= FeatureEdge(sideLines[i]);
sideFEdges[i].numberOfElements = NumELhps;
if (i<3) {
sideLines[i+6] = GuideLine(myPoints[i+12],myPoints[i+15],0);
sideFEdges[i+6] = FeatureEdge(sideLines[i+6]);
sideFEdges[i+6].numberOfElements = NumELhps;
}
}

sideLines[9] = GuideLine(P_hp1top3,P_hp1bot3,0);
sideFEdges[9]= FeatureEdge(sideLines[9]);
sideFEdges[9].numberOfElements = NumELhps;
sideLines[10] = GuideLine(P_hp1top4,P_hp1bot4,0);
sideFEdges[10]= FeatureEdge(sideLines[10]);
sideFEdges[10].numberOfElements = NumELhps;


// Refine mesh at middle of heave plates
Curve_bothp2 = CreateCircleFromPlaneAndRadius(P23,Vector3d(0,0,1),chp); 

P13_1 = P13.copyTranslate(Vector3d(-chp,0,0));
P13_2 = P13.copyTranslate(Vector3d(chp,0,0));
P13_3 = P13.copyTranslate(Vector3d(0,chp,0));
Curve_bothp11 = GuideArcElliptic(P13, P13_2, P13_3, true);
Curve_bothp12 = Curve_bothp11.copyRotate(P13,Vector3d(0,0,1),90*1);
Curve_tophp2 = Curve_bothp2.copyTranslate(Vector3d(0,0,hhpM));
Curve_tophp11 = Curve_bothp11.copyTranslate(Vector3d(0,0,hhpM));
Curve_tophp12 = Curve_bothp12.copyTranslate(Vector3d(0,0,hhpM));

FEdge_bothp2 = FeatureEdge(Curve_bothp2);
FEdge_bothp2.meshDensity = denshp_botc; //numberOfElements = NumElWL2;

FEdge_tophp2 = FeatureEdge(Curve_tophp2);
FEdge_tophp2.meshDensity = denshp_topc;

FEdge_bothp11 = FeatureEdge(Curve_bothp11);
FEdge_bothp11.meshDensity = denshp_botc; //numberOfElements = NumElWL1;

FEdge_bothp12 = FeatureEdge(Curve_bothp12);
FEdge_bothp12.meshDensity = denshp_botc;

FEdge_tophp11 = FeatureEdge(Curve_tophp11);
FEdge_tophp11.meshDensity = denshp_topc;

FEdge_tophp12 = FeatureEdge(Curve_tophp12);
FEdge_tophp12.meshDensity = denshp_topc;

// Add wet surfaces to the set Wet_Surface
Wet_Surface = Set();
Wet_Surface.add(Pl1);
Wet_Surface.add(Pl1_top);
Wet_Surface.add(Pl2);
Wet_Surface.add(Pl2_top);
Wet_Surface.add(Pl4);
Wet_Surface.add(Pl11);
Wet_Surface.add(Pl_hp1bot);
Wet_Surface.add(Pl_hp2bot);
Wet_Surface.add(Pl_hp1s0);
Wet_Surface.add(Pl_hp1s1);
Wet_Surface.add(Pl_hp1s2);
Wet_Surface.add(Pl_hp1s3);
Wet_Surface.add(Pl_hp2s0);
Wet_Surface.add(Pl_hp2s1);
Wet_Surface.add(Pl_hp2s2);
Wet_Surface.add(Pl_hp2s3);
Wet_Surface.add(Pl_hp2s4);
Wet_Surface.add(Pl_hp2s5);

// Define wet part of hull, for hydro pressure
WS1 = WetSurface();
// Make sure normal vector points out into water
Wet_Surface.front.wetSurface = WS1;

// Create load case for HydroD
LC1 = DummyHydroLoadCase(WS1);
LC1.setFemLoadcase(1); // unimportant or 1 for hydroD
LC1.designCondition(lcOperating); // Choose design condition, operation now
LC1.generateAppliedLoads();  

// Create analysis in order to mesh
Mesh_Panels = Analysis(true); // Construct an empty analysis
Mesh_Panels.add(MeshActivity()); // Create new mesh activity
Mesh_Panels.setActive();
Mesh_Panels.step(1).subset = Wet_Surface; // Creates less elements, dont mesh lines!
Mesh_Panels.step(1).regenerateMeshOption = anAlwaysRegenerateMesh;
Mesh_Panels.step(1).nodeNumberFromJointName = true;
SimplifyTopology(); // Could be used!

// Create mesh
Mesh_Panels.execute();

// Export file for use in HydroD
ExportMeshFem().DoExport("T1.FEM");

