// HydroD script for WindFloat
// Written by Vetle Kallåk 2020

// Input data
// MATLAB WRITES LINE 6-19 and 26!!
g = 9.8066 m/s^2; // Gravity
rho = 1025 Kg/m^3; // Sea water density
d = 320 m; // Depth

totMass = 7973454.7099 Kg; // Modeled mass
COG_1 = -0.26058 m; // Center of gravity
COG_2 = 0 m; // Center of gravity
COG_3 = 5.9533 m; // Center of gravity
r_1 = 47.7807 m; // Radius of gyration
r_2 = 47.6535 m; // Radius of gyration
r_3 = 38.7701 m; // Radius of gyration
r_ij1 = 0 m; // Specific product of inertia
r_ij2 = -25.7877 m; // Specific product of inertia
r_ij3 = 0 m; // Specific product of inertia


// Wave directions
DirectionSet1 = DirectionSet(Array(0 deg));

// Wave frequenies
FrequencySet1 = FrequencySet(FrequencyTypeFrequency, Array(0.05 rad/s,0.1 rad/s,0.15 rad/s,0.2 rad/s,0.25 rad/s,0.3 rad/s,0.35 rad/s,0.4 rad/s,0.45 rad/s,0.5 rad/s,0.55 rad/s,0.6 rad/s,0.65 rad/s,0.7 rad/s,0.75 rad/s,0.8 rad/s,0.85 rad/s,0.9 rad/s,0.95 rad/s,1 rad/s,1.05 rad/s,1.1 rad/s,1.15 rad/s,1.2 rad/s,1.25 rad/s,1.3 rad/s,1.4 rad/s,1.5 rad/s,1.6 rad/s,1.7 rad/s,1.8 rad/s,1.9 rad/s,2 rad/s));


// Loaction 
Location1 = Location();
Location1.setDepth(d);
Location1.gravity = g;
Location1.air().density = 1.225 Kg/m^3;
Location1.air().kinematicViscosity = 1.462e-005 m^2/s;
Location1.water().density = rho;
Location1.water().kinematicViscosity = 1.19e-006 m^2/s;
Location1.seabed().normaldirection = Vector3d(0 m,0 m,1 m);

// Condition
Condition1 = FrequencyDomain(Location1);
Condition1.waterSurface().directionSet = DirectionSet1;
Condition1.waterSurface().frequencySet = FrequencySet1;
Condition1.water().setNoCurrent();

// Hydro Model
HydroModel1 = HydroModel(HydroModelFloating); // Floating hydromodel
HydroModel1.setColumnStabilized(false); // For stability rule checks
HydroModel1.setBaselineZPos(0 m); 
HydroModel1.setAPXPos(0 m);
HydroModel1.setFPXPos(100 m); //Baseline, AP and FP only relevant for Wasim
// Rest is for reporting in displacement table
HydroModel1.clearReportMetaCenterRotationAxisAzim();
HydroModel1.addReportMetaCenterRotationAxisAzim(0 deg);
HydroModel1.addReportMetaCenterRotationAxisAzim(90 deg);
HydroModel1.clearReportHeelTrimCombinations();
HydroModel1.addReportHeelTrimCombination(0 deg, 0 deg);
HydroModel1.clearReportZWaterlines();
// Make for loop for waterlines
for (zWater = -30; zWater <= 20; zWater += 0.5) {
HydroModel1.addReportZWaterline(zWater);
}

// Panel model
PanelModel1 = PanelModel(HydroModel1, ElementEnumsFEMFile, "T1.FEM", true, false); // FEMfile, last: xz and yz symmetry

// Loading condtion
LoadingCondition1 = LoadingCondition(HydroModel1, 0 m, 0 deg, 0 deg); // Model changes in draft, heel and trim for this loading condition
LoadingCondition1.setByDraft(false); // Method to define trim and draft by mass
LoadingCondition1.interpolateDampingMatrices(false); // Same damping matrix for all wave freq
LoadingCondition1.addDampingMatricesToWadam(true); // 


// Mass model 
MassModel1 = MassModel(LoadingCondition1, MassModelSpecified);
MassModel1.setUserMassCoordinateSystem(GlobalCoordinateSystem); //model adjusted after draft, heel and trim for LC
MassModel1.setTotalMass(totMass);
MassModel1.setCOG(Point(COG_1,COG_2,COG_3));
MassModel1.setRadiusGyration(Vector3d(r_1,r_2,r_3));
MassModel1.setSpecificProductInertia(r_ij1, r_ij2, r_ij3);
// Compartments in model
MassModel1.addTankMass(true); // add tank fluid to total weight
MassModel1.internalDynamics(false); // Dynamics of internal fluid 
MassModel1.updateStiffnessWithFreeSurfaceEffect(true); // from internal tanks


// Wadam Run
WadamRun1 = WadamRun();
// Input
WadamRun1.setHydroModel(HydroModel1);
WadamRun1.useMultiBody(false);
WadamRun1.setLoadingCondition(LoadingCondition1);
WadamRun1.setEnvironmentData(Condition1);
WadamRun1.useSeaState(false);

// Execution directives
WadamRun1.dataCheck(false);
WadamRun1.setAnalysisType(WadamRunGlobalResponse);
// Constants
WadamRun1.setToleranceWaterLine(5);
WadamRun1.setToleranceCOG(5);
WadamRun1.setCharacteristicLength(100 m);

WadamRun1.calculateDrift(false);
WadamRun1.waveDriftDamping(false);
WadamRun1.calculateRollDamping(false);

// Equation solver
WadamRun1.setSolverType(WadamRunDirectSolver);
WadamRun1.setMaxMatrixDimension(10000); // 8Gb free memory gives 32000 panels
// Print
//WadamRun1.setPrintType(WadamRunNormalPrint);
WadamRun1.setPrintType(WadamRunModelDataPrint); // Print pressure
// Result files
WadamRun1.setResponseFileType(WadamRunSIFFormatted);
WadamRun1.calculateEigenvalues(true);
WadamRun1.defineGlobalResponseReferencePoint(false);
WadamRun1.setGlobalResponseReferencePoint(Point(0 m,0 m,0 m));
WadamRun1.sumFrequencyResults(false);
WadamRun1.differenceFrequencyResults(false);
// Advanced
WadamRun1.setSingularityType(WadamRunAnalyticalSingularity); // Analytical give better result, use more time
WadamRun1.setIntegrationType(WadamRunOneNodeGauss); // Faster, but less precise than 4node
WadamRun1.setPanelDimensionType(WadamRunMaximumDiagonalPanelDimension);
WadamRun1.useWadamMassCalculation(false);
WadamRun1.removeIrrFrequencies(true); // BC problem
WadamRun1.setIncludeForwardSpeedEffect(false);
WadamRun1.saveTempWamitFiles(false); // 
WadamRun1.stopBeforePotenExecution(false);
WadamRun1.bypassPotenExecution(false);
WadamRun1.stopBeforeFirstForceExecution(false);
WadamRun1.bypassFirstForceExecution(false);
WadamRun1.stopBeforeSecondForceExecution(false);
WadamRun1.bypassSecondForceExecution(false);
WadamRun1.useSaveRestart(false);

// Har med streching of free surface, only for determenistic calc. wave amp specified! For Load transfer
WadamRun1.setWaterlinePanelMethod(WadamRunNoPanelPressureAdjustment);
WadamRun1.setLoadTransferLinearInterpolationXmin(0);
WadamRun1.setLoadTransferLinearInterpolationXmax(0);
WadamRun1.autoOwerwriteExistingResultFiles(false);
// Output
WadamRun1.specifyOutputDirectory(false, false);
WadamRun1.setOutputDirectory("");
WadamRun1.setOutputPrefix("");
WadamRun1.moveResultFiles(false);
WadamRun1.useRunNameAsPrefix(false);


WadamRun1.execute();
