% Dimensions, mass and hydrostatic data for WindFloat concept
% Written by Vetle Kallaak

function [M, C, elem, alpha, mdata] = hydrostatic(env,p)
rho = env.rho;
g = env.g;
%% Elements of the structure
elem = struct('name',[],'L',[],'A',[],'Vwet',[],'mass',[],'COG',[]);
% Columns
elem(1).name = 'column';
elem(1).L = p.FB1+p.draft-p.hhp;
elem(1).A = pi*(p.diaCol/2)^2;
elem(1).Vwet = elem(1).A * (p.draft-p.hhp);
elem(2:3) = elem(1);
Vcol_wetM = elem(1).A * (p.draft-p.hhpM) * p.numCol; % Sesam volume
% Heave plates
elem(4).name = 'hp';
elem(4).L = p.hhp;
elem(4).A = (3*sqrt(3)/2 * p.lhp^2);
elem(4).Vwet = elem(4).A * elem(4).L;
elem(5:6) = elem(4);
VhpM = elem(4).A * p.hhpM * p.numCol; % Sesam volume
% Beams and pontoons
elem(7).name = 'beam';
elem(7).L = p.dcc - p.diaCol;
elem(7).A = pi*(p.diaBeam/2)^2;
elem(7).Vwet = 0;
elem(8:12) = elem(7);
[elem(10:12).name] = deal('pontoon');
[elem(10:12).Vwet] = deal(elem(10).A * elem(10).L);
%% Bracing
elem(13).name = 'bracing';
alpha = atan((-p.COGz_pon+p.COGz_beam)/(p.dcc/2)) % Angle between pontoon and bracing
diaBracHor = p.diaBracing/sin(alpha); % Horizontal cross section, max dia
% Corrected alpha
alpha = atan((-p.COGz_pon-p.diaBeam/2+p.COGz_beam)/(p.dcc/2-diaBracHor/2)) 
notBracing = p.diaCol/2 / cos(alpha);
elem(13).L = sqrt((-p.COGz_pon-p.diaBeam/2+p.COGz_beam)^2 + ...
    (p.dcc/2-diaBracHor/2)^2)-notBracing;
lBracW = (-p.COGz_pon - p.diaBeam/2)/sin(alpha); % Wet length of bracing
elem(13).A = pi*(p.diaBracing/2)^2;
elem(13).Vwet = elem(13).A * lBracW;
elem(14:18) = elem(13); % 6 bracings in total

%% Volume and mass displacement
% GeniE structure
mdata.VdispM = Vcol_wetM + VhpM;
mdata.massM = mdata.VdispM*rho;
% Entire structure
VdispTot = sum([elem.Vwet]);
massTot = VdispTot*rho;
mdata.VdispTot = VdispTot;
mdata.massTot = massTot;

%% Steel mass
massSteel = p.massFracSteel * massTot; % Steel mass of hull
mdata.massSteel = massSteel;
% Volume of steel, for equally distribution of weight
VSCol_p = pi*(p.diaCol*p.t-p.t^2)*(p.FB1+p.draft-p.hhp-p.t);
VSCol_top = pi*(p.diaCol/2)^2*p.t;
VSCol = (VSCol_p + VSCol_top) * p.numCol;
VSBeam = pi*(p.diaBeam*p.t-p.t^2)*(p.dcc-p.diaCol)*p.numCol;
VSPon = VSBeam;
VSBracing = pi*(p.diaBracing*p.t-p.t^2)*elem(13).L*p.numCol*2;
VShp = elem(4).A*p.thp*p.numCol;
[elem(1:3).mass] = deal(massSteel * VSCol/(VSCol+VSPon+VSBeam+VSBracing+VShp)/p.numCol);
[elem(4:6).mass] = deal(massSteel * VShp/(VSCol+VSPon+VSBeam+VSBracing+VShp)/p.numCol);
[elem(7:9).mass] = deal(massSteel * VSBeam/(VSCol+VSPon+VSBeam+VSBracing+VShp)/p.numCol); 
[elem(10:12).mass] = deal(massSteel * ...
    VSPon/(VSCol+VSPon+VSBeam+VSBracing+VShp) / p.numCol);
[elem(13:18).mass] = deal(massSteel * ...
    VSBracing/(VSCol+VSPon+VSBeam+VSBracing+VShp) / (2*p.numCol));

%% Ballast and WT mass
elem(19).name = 'ballast1';
elem(20).name = 'ballast2';
massBallast = massTot-massSteel-p.WT.masstot; % Total req. ballast
massFracBallast2 = (1-p.massFracBallast1)/2;
mdata.massBallast = massBallast;
elem(19).Vwet = 0;
elem(19).mass = massBallast*p.massFracBallast1;
elem(20).Vwet = 0;
elem(20).mass = massBallast*massFracBallast2;
% Height of ballast
elem(19).L = ((elem(19).mass / p.rhoB) / (pi*(p.diaCol/2-p.t)^2));
elem(20).L = ((elem(20).mass / p.rhoB) / (pi*(p.diaCol/2-p.t)^2));
elem(21) = elem(20);
% Wind turbine
elem(22).name = 'WT';
elem(22).Vwet = 0;
elem(22).mass = p.WT.masstot;

%% Center of gravity
% elem(22).mass=0; % To get data w/o WT (wrong draft etc.)
% Columns
COGz_col = (VSCol_p*(p.FB1-p.t-(p.FB1+p.draft-p.hhp-p.t)/2)+ ...
    VSCol_top*(p.FB1-p.t/2))/(VSCol_p+VSCol_top);
elem(1).COG = [p.dcc/2/cos(pi/6), 0, COGz_col];
elem(2).COG = [-p.dcc/2*tan(pi/6), p.dcc/2, COGz_col];
elem(3).COG = [-p.dcc/2*tan(pi/6), -p.dcc/2, COGz_col];
% Heave plates
elem(4).COG = [p.dcc/2/cos(pi/6), 0, -1*(p.draft-p.hhp/2)];
elem(5).COG = [-p.dcc/2*tan(pi/6), p.dcc/2, -1*(p.draft-p.hhp/2)];
elem(6).COG = [-p.dcc/2*tan(pi/6), -p.dcc/2, -1*(p.draft-p.hhp/2)];
% Beams
elem(7).COG = [(elem(1).COG(1)+elem(2).COG(1))/2, ...
(elem(1).COG(2)+elem(2).COG(2))/2, p.COGz_beam];
elem(8).COG = [(elem(1).COG(1)+elem(2).COG(1))/2, ...
    -(elem(1).COG(2)+elem(2).COG(2))/2, p.COGz_beam];
elem(9).COG = [elem(2).COG(1), 0, p.COGz_beam];
% Pontoons 
elem(10).COG = [(elem(1).COG(1)+elem(2).COG(1))/2, ...
    (elem(1).COG(2)+elem(2).COG(2))/2, p.COGz_pon];
elem(11).COG = [(elem(1).COG(1)+elem(2).COG(1))/2, ...
    -(elem(1).COG(2)+elem(2).COG(2))/2, p.COGz_pon];
elem(12).COG = [elem(2).COG(1), 0, p.COGz_pon];
% Bracing
COGz_brac = p.COGz_pon + p.diaBeam/2 + elem(13).L/2*sin(alpha);
elem(13).COG = [elem(10).COG(1), elem(10).COG(2), COGz_brac];
elem(14).COG = [elem(10).COG(1), elem(10).COG(2), COGz_brac];
elem(15).COG = [elem(11).COG(1), elem(11).COG(2), COGz_brac];
elem(16).COG = [elem(11).COG(1), elem(11).COG(2), COGz_brac];
elem(17).COG = [elem(12).COG(1), elem(12).COG(2), COGz_brac];
elem(18).COG = [elem(12).COG(1), elem(12).COG(2), COGz_brac];
% Ballast
elem(19).COG = [p.dcc/2/cos(pi/6), 0, ...
    p.COGz_ballastBot + elem(19).L/2];
elem(20).COG = [-p.dcc/2*tan(pi/6), p.dcc/2, p.COGz_ballastBot + elem(20).L/2];
elem(21).COG = [-p.dcc/2*tan(pi/6), -p.dcc/2, p.COGz_ballastBot + elem(21).L/2];
% WT
elem(22).COG = [elem(1).COG(1)+p.WT.COGx, elem(1).COG(2)+p.WT.COGy, p.FB1+p.WT.COGz];

%% Mass matrix data
COGmom = zeros(1,3); % Global center of gravity, mass moment
L_ij = zeros(1,3); % For specific product of inertia
% Local inertia of main contributors 
I(1) = p.numCol*(elem(1).mass*(0.5*(p.diaCol/2)^2+1/12*elem(1).L^2)...
    +1/4*elem(4).mass*(elem(4).A/pi));
I(2) = I(1);
I(3) = p.numCol*(elem(1).mass*(p.diaCol/2)^2+0.5*elem(4).mass*(elem(4).A/pi) ...
    + 2*elem(7).mass*(0.5*(p.diaBeam/2)^2+elem(7).L^2/12)...
    + 2*elem(13).mass*elem(13).L^2/12*(sin(pi/2-alpha))^2 );
for i = 1:length(elem) % All elements
   for d=1:3
      COGmom(d) =  COGmom(d) + elem(i).mass * elem(i).COG(d);
   end
   % Parellel axis theorem
   I(1) = I(1) + elem(i).mass * (elem(i).COG(2)^2+elem(i).COG(3)^2);
   I(2) = I(2) + elem(i).mass * (elem(i).COG(1)^2+elem(i).COG(3)^2);
   I(3) = I(3) + elem(i).mass * (elem(i).COG(1)^2+elem(i).COG(2)^2);
   L_ij(1) = L_ij(1) + elem(i).mass * elem(i).COG(1)*elem(i).COG(2);
   L_ij(2) = L_ij(2) + elem(i).mass * elem(i).COG(1)*elem(i).COG(3);
   L_ij(3) = L_ij(3) + elem(i).mass * elem(i).COG(2)*elem(i).COG(3);
end
mdata.COG = COGmom/massTot;
% Radii of gyration, r=(r_xx, r_yy, r_zz)
mdata.r = sqrt((1/massTot)*I);
% Specific product of inertia, r_ij=(r_xy, r_xz, r_yz)
mdata.r_ij = - sqrt((1/massTot)*L_ij) .* sign(L_ij);
% Calculate mass matrix
[M] = massMat(massTot, mdata.COG, mdata.r, mdata.r_ij);

%% Hydrostatic restoring data
% Center of buoyancy
COB=zeros(1,3);
for i=1:length(elem)
    COB(1) = COB(1) + elem(i).Vwet*elem(i).COG(1);
    COB(2) = COB(2) + elem(i).Vwet*elem(i).COG(2);
    if i<=3 % Columns
        COB(3) = COB(3) + elem(i).Vwet*(-p.draft+p.hhp)/2;
    elseif i >= 13 % Bracing 
        COB(3) = COB(3) + elem(i).Vwet*((p.COGz_pon+p.diaBeam/2)/2);
    else
        COB(3) = COB(3) + elem(i).Vwet*elem(i).COG(3);
    end
end
COB = COB/VdispTot;
mdata.COB = COB;

% Waterplane inertia
Icol = pi*p.diaCol^4/64;
Iwp(1) = p.numCol*Icol + 2*elem(2).COG(2)^2*elem(2).A + ... 
    elem(1).COG(2)^2*elem(1).A + pi*p.diaBracing^2/(4*sin(alpha))*2* ...
    ((elem(12).COG(2)+diaBracHor/2+lBracW*cos(alpha))^2 +...
    (elem(10).COG(2)+diaBracHor/2*sin(pi/6)+lBracW*cos(alpha)*sin(pi/6))^2 +...
    (elem(10).COG(2)-diaBracHor/2*sin(pi/6)-lBracW*cos(alpha)*sin(pi/6))^2 ); % Ixx
Iwp(2) = p.numCol*Icol + 2*elem(2).COG(1)^2*elem(2).A + ... 
    elem(1).COG(1)^2*elem(1).A + ...
    pi*p.diaBracing^2/(4*sin(alpha))*2*(elem(12).COG(1)^2 +...
    (elem(10).COG(1)+diaBracHor/2*cos(pi/6)+lBracW*cos(alpha)*cos(pi/6))^2+...
    (elem(10).COG(1)-diaBracHor/2*cos(pi/6)-lBracW*cos(alpha)*cos(pi/6))^2); % Iyy

BM = Iwp./ VdispTot;  
GM = BM - (mdata.COG(3)-COB(3)); % 1: transverse, 2: longitudinal
mdata.GM = GM;
% Calculate hydrostatic restoring matrix
C = zeros(6,6);
C(3,3) = rho*g*(elem(1).A + pi*p.diaBracing^2/(4*sin(alpha))*2) * p.numCol;
C(3,4) = rho*g* elem(1).A*(elem(1).COG(2)+elem(2).COG(2)+elem(3).COG(2)); 
C(4,3) = C(3,4);
C(3,5) = -rho*g* elem(1).A*(elem(1).COG(1)+elem(2).COG(1)+elem(3).COG(1));
C(5,3) = C(3,5);
C(5,4) = -rho*g* (elem(1).A*(elem(1).COG(1)*elem(1).COG(2)+ ...
    elem(2).COG(1)*elem(2).COG(2)+elem(3).COG(1)*elem(3).COG(2))); 
C(4,5) = C(5,4);
C(4,6) = -rho*g*VdispTot*COB(1) + massTot*g*mdata.COG(1);
C(5,6) = -rho*g*VdispTot*COB(2) + massTot*g*mdata.COG(2);
C(4,4) = rho*g* VdispTot * GM(1);
C(5,5) = rho*g* VdispTot * GM(2);
end
