% Upscaled WindFloat 10 MW structure  
% Written by Vetle Kallaak
clear variables; close all;

% Environment
env.rho = 1025; % Sea water density
env.g = 9.80665; % Gravity
env.d = 320; % Depth [m]
env.amp = 1; % Wave amplitude [m]
env.beta = 0; % Wave heading angle [rad]
% Parameters
p.FB1 = 10; % Column freeboard
% Wind turbine
[p.WT] = WTdata(p.FB1);

p.numCol = 3; % Number of columns
p.draft = 20; % Floater draft 
p.diaCol = 11.5; % Column diameter
p.dcc = 68.5; % Center-center distance for columns
p.lhp = 17.5; % Heave plate length of plate edge (hexagon)
p.diaBeam = 2.8; % Truss main beam diameter
p.diaBracing = 2.0; % Truss bracing diameter

% Unknown variables, guessed
p.COGz_pon = -p.draft+p.diaBeam; % Mean position of pontoons
p.COGz_beam = p.FB1 - p.diaBeam; % Mean position of bracing
p.hhp = 0.05; % Height of heave plate (Kvittem)
p.hhpM = 0.4; % Height of heave plates modeled in GeniE
p.FBM = 1; % Modeled freeboard in GeniE
p.massFracSteel = 0.20; % Steel mass, fraction of total mass (From Erin)
p.t = 0.03; % Steel plates thickness
p.thp = 0.05; % Heave plate thickness

% Adjustable ballast for right output
p.massFracBallast1 = 0.18;
p.rhoB = env.rho; % Density of ballast
p.COGz_ballastBot = -p.draft + p.hhp + p.t; % Ballast floor

p.tilt = 0 * pi/180; % Pitch tilt angle

% GeniE mesh, depend a bit on wave freq and dimensions
mesh.gensize = 1.4; % General mesh element size
mesh.maxsize = 2.2; % Max mesh size, for under hp

% Calculate RAO and Morison forces
sesam.workspace = '10Mw_opti1';
newrun = false;

% Criteria 
crit.eta5max = (10*pi/180); % Max allowable pitch angle
crit.T33min = 20; % Min natural period in heave
crit.DhpDcmin = 1.6; % Min diameter ratio, hp/column

% Wave frequencies
w1 = 0.05:0.05:1.3;
w2 = 1.4:0.1:2;

% HydroD workspaces
sesam.wadam1 = [sesam.workspace '\WadamRun1']; 
sesam.wadam2 = [sesam.workspace '\WadamRun2']; % Extra damping
WaveHeadInd = 1; % Index of the wave heading for which to calc everything
figPosition = [100 100 1000 650];

interpolPoints = 16;

%% Hydrostatic calculations and structure proporties 
[M, C, elem, alpha, mdata] = hydrostatic(env, p);
mdata
C(1,1) = p.WT.Ft/(0.15*env.d); % Mooring stiffness 

%% Wave environment
% Waves
clear waves
waves(:,1) = [w1 w2];
waves(:,2) = 2*pi./waves(:,1);
waves(:,3) = 2*pi*env.g./waves(:,1).^2;
maxElemSize = min(waves(:,3))/6 % Max element diagonal

%% Control of input parameters
[T33, A33, check] = control(env,p,elem,alpha,C,mdata,crit);
choice = menu('Continue?','Yes','No');
if choice == 2 || choice == 0
    return;
end

%% Sesam
mesh.numElWL2 = ceil(pi*p.diaCol/mesh.gensize); % Elemenents on WL column 2
if newrun
    Bex = zeros(6,6);
    check = writeSesamFiles(sesam, p, mesh, env, waves, C, mdata, Bex, 1);
    if check
        cont = runSesam(p, sesam, 1); % WadamRun 1
        if ~cont % Continue
            return;
        end
    end
end

%% Read Wadam.LIS file, results from Wadam
% Read results from Wadam
[MASS, RESTORING, DAMPING, ADDMASS, WAVEEX, MOTIONS, WaveDat, WaveDir] = ...
    WadamLIS(sesam.wadam1);
% Rewrite potential excitation force and RAO from Wadam file
for i=1:length(WaveDat(:,1))
    F_pot(1:6,i) = complex(WAVEEX(:,1,WaveHeadInd,i),WAVEEX(:,2,WaveHeadInd,i));
    etaW(1:6,i) = complex(MOTIONS(:,1,WaveHeadInd,i),MOTIONS(:,2,WaveHeadInd,i));
end
waveW(1,:) = WaveDat(:,5); % Wave frequencies 

%% Loads on Morison elements
[etaM1, etaM0, FtotwI, FtotwD, Atot_mor, Btot_mor, F_hp1, B_hp1] = ...
includeMorisonElem(p, elem, alpha, env, M, C,...
waveW, ADDMASS, DAMPING, F_pot);
    
%% Calculate RAOs
% Calculate RAO, solving the 6DOF equation of motion with Wadam results
[etaWown] = calcRAO(WaveDat(:,5), MASS, ADDMASS, DAMPING, RESTORING, F_pot);
% Interpolate values for better solution
% Linear interpolation of A and B matrices
[A_pothr] = linInterpolation(ADDMASS, 3, interpolPoints);
[B_pothr] = linInterpolation(DAMPING, 3, interpolPoints);
[Btot_morhr] = linInterpolation(Btot_mor, 3, interpolPoints);
[B_hp1hr] = linInterpolation(B_hp1, 3, interpolPoints);

waveWhr = linInterpolation(waveW, 2, interpolPoints); 
% Spline interpolation of forces for better fit
for r = 1:6 % Splining
    F_pothr(r,:) = interp1(waveW,F_pot(r,:),waveWhr,'spline');
    F_totwIhr(r,:) = interp1(waveW,FtotwI(r,:),waveWhr,'spline');
    F_totwDhr(r,:) = interp1(waveW,FtotwD(r,:),waveWhr,'spline');
    F_hp1hr(r,:) = interp1(waveW,F_hp1(r,:),waveWhr,'spline');
end
Atot = A_pothr + Atot_mor; % Total added mass matrix for structure
Btot = B_pothr + Btot_morhr; % Total damping for structure
Ftot = F_pothr + F_totwIhr + F_totwDhr; % Total excitation loads
% RAO from wadam with interpolated values
[etaWhr] = calcRAO(waveWhr, MASS, A_pothr, B_pothr, RESTORING, F_pothr);
% Final RAO
[eta1] = calcRAO(waveWhr, M, Atot, Btot, C, Ftot);

%% Natural frequencies
for dof=1:6 %dof
    Tn_all(:,1) = 2*pi*sqrt((M(dof,dof)+Atot(dof,dof,:))/(C(dof,dof)));
    Tn_all(:,2) = 2*pi./waveWhr(:);
    [~, waven_index(dof)] = min(abs(Tn_all(:,1) - Tn_all(:,2)));
    Tn(dof,1) = Tn_all(waven_index(dof),1); % Natural period
    Tn(dof,2) = 2*pi/Tn(dof,1); % Natural frequency
end
Tn
waven_index

%% Additional damping to HydroD
% For calculating more correct motions, read pressure panels, get more
% correct radiation pressure
if newrun
    Bex = Btot_morhr(:,:,waven_index(3))
    check = writeSesamFiles(sesam, p, mesh, env, waves, C, mdata, Bex, 2);
    if check
        cont = runSesam(p, sesam, 2); % WadamRun 2
    end
end
% Read results from Wadam
[MASS1, RESTORING1, DAMPING1, ADDMASS1, WAVEEX1, MOTIONS1, WaveDat1, WaveDir1] = ...
    WadamLIS(sesam.wadam2);
% Rewrite potential excitation force and RAO from Wadam file
for i=1:length(WaveDat1(:,1))
    etaW_Bex(1:6,i) = complex(MOTIONS1(:,1,WaveHeadInd,i),MOTIONS1(:,2,WaveHeadInd,i));
end
[etaW_Bexown] = calcRAO(WaveDat1(:,5), MASS1, ADDMASS1, DAMPING1, RESTORING1, F_pot);

%% Calculate internal loads at cross section
[numElem, Fp, Fc, F_Mhp1, FI, Fcs] = internalLoads(sesam, p, elem, env, waveW, ...
    waveWhr, F_hp1hr, B_hp1hr, eta1, ADDMASS, DAMPING, etaM1, etaW, etaW_Bex, F_pot);

%% Save results
if newrun
    %% Save entire workspace
    clear aa ac bb bc c choice cont dof ee ff i f plotFig r Tn_all
    save(['C:\Users\Vetle\OneDrive  NTNU\Emner\MASTER\MATLAB\Results\' sesam.workspace])
end
fprintf('\nFinished!\n\n');
