% Calculation of forces on slender elements by using Morison's equation
% And viscous forces on heave plates
% Written by Vetle Kallaak

function [FtotwaveI,FtotwaveD, Atot_mor, Btot_mor, F_hp1, B_hp1] = ...
    morison(first, waveW, eta, env, p, h_approx, COC, coord)
movingStructure = true;
% coord - Description of Morison elements
% Rows: Morison element
% Columns: (x,y,z) start and then end of Morison element (keep normal
% vector positiv). Column 7: Diameter, 8: C_a, 9:C_D

% Environment
g = env.g; % Gravitational acceleration
rho = env.rho; % Sea water density [kg/m^3]
d = env.d; % Depth
A = env.amp; % Wave amplitude
beta = env.beta; % Wave angle

% Calculate wave numbers and wave lengths
Nfreq = length(waveW);
k_all = zeros(Nfreq,1);
for cc=1:Nfreq
    w=waveW(cc);
    k1 = w^2/g; %Wave number, assumed deep water
    k_n=10;
    k_n1 = k1; 
    while abs(k_n1 - k_n) > 0.00001
        % Newton's method
        k_n = k_n1;
        k_n1=k_n - (w^2-k_n*g*tanh(k_n*d))/(-g*tanh(k_n*d)-k_n*g*d*(1/(cosh(k_n*d))^2));
    end
    k_all(cc,1)=k_n1;
end

FwaveI = zeros(6,Nfreq,length(coord(:,1))); % Wave inertia loads
FwaveD = zeros(6,Nfreq,length(coord(:,1))); % Wave drag loads
B_mor = zeros(6,6,Nfreq,length(coord(:,1))); % Damping matrix
A_mor = zeros(6,6,length(coord(:,1))); % Added mass matrix

for el=1:length(coord(:,1)) % Morison elements
    D = coord(el,7); % Diameter of element
    C_a = coord(el,8); % Added mass coefficient 
    C_D = coord(el,9); % Drag coefficient
    % Calculate length of element
    L = sqrt((coord(el,4)-coord(el,1))^2 + (coord(el,5)-coord(el,2))^2 + ...
        (coord(el,6)-coord(el,3))^2);
    if L == 0 % Heave plate
        steps = 1; % Calculate only for center of heave plate
        h = 0; % Element step length
        if p.tilt ~= 0
            n = [cos(p.tilt) 0 -sin(p.tilt)];
            % Projection matrix 
            P = [n(2)^2+n(3)^2, -n(1)*n(2), -n(1)*n(3);
                -n(1)*n(2), n(1)^2+n(3)^2, -n(2)*n(3);
                -n(1)*n(3), -n(2)*n(3), n(1)^2+n(2)^2];
        else
            n = [0 0 0]; % Normal vector of element
            P = zeros(3,3);
            P(3,3) = 1; % Only want force in z-direction
        end
    else
        steps = round(L/h_approx);
        h = L/steps; % Element step length
        % Calculate unit vector of element along x direction
        n(1) = (coord(el,4)-coord(el,1))/L;
        % Calculate unit vector of element along y direction
        n(2) = (coord(el,5)-coord(el,2))/L;
        % Calculate unit vector of element along z direction
        n(3) = (coord(el,6)-coord(el,3))/L;
        % Projection matrix 
        P = [n(2)^2+n(3)^2, -n(1)*n(2), -n(1)*n(3);
            -n(1)*n(2), n(1)^2+n(3)^2, -n(2)*n(3);
            -n(1)*n(3), -n(2)*n(3), n(1)^2+n(2)^2];
    end
    dFwaveInertia = zeros(6,Nfreq);
    dFwaveDrag = zeros(6,Nfreq);
    dB_mor = zeros(6,6,Nfreq);
    dA_mor = zeros(6,6);
    for j=1:steps % steps along element
        % Coordinates to center of part
        x = coord(el,1) + h*n(1)*(j-0.5);
        y = coord(el,2) + h*n(2)*(j-0.5);
        z = coord(el,3) + h*n(3)*(j-0.5);        
        % For rotation to translation and force to moment
        S = [0, z-COC(3), -(y-COC(2));
            -(z-COC(3)), 0, x-COC(1);
            y-COC(2), -(x-COC(1)), 0];
        for cc=1:Nfreq % All wave frequencies
            w = waveW(cc); % Wave frequency
            k = k_all(cc,1); % Wave number

            % Wave velocity field, U=[u;v;w]
            U = [w*A*(cosh(k*(z+d))/sinh(k*d))*exp(-1i*k*(x*cos(beta)+...
                y*sin(beta)))*cos(beta);
                w*A*(cosh(k*(z+d))/sinh(k*d))*exp(-1i*k*(x*cos(beta)+...
                y*sin(beta)))*sin(beta);
                1i*w*A*(sinh(k*(z+d))/sinh(k*d))*exp(-1i*k*(x*cos(beta)+...
                y*sin(beta)))];
            % Linearization of drag term
            if movingStructure
                K = 8/(3*pi) * abs(P*(U-1i*w*[eye(3) S]*eta(:,cc))); 
            else
                K = abs(P*U);
            end
            if L == 0 % Heave plate
               % Wave drag force
               dFwD = 0.5*C_D*rho*pi*D^2/4* K.*(P*U);
               % Morison damping matrix, related to drag force
               dB = 0.5*C_D*rho*pi*D^2/4* K.* (P*[eye(3) S]);
            else
                % Wave drag force
                dFwD = 0.5*C_D*rho*h*D* K.*(P*U);
                % Morison damping matrix, related to drag force
                dB = 0.5*C_D*rho*h*D* K.* (P*[eye(3) S]);
                % Wave inertia force and moment
                if first
                    dFwI = 1i*w*rho*h*pi*D^2/4*(1+C_a)*(P*U);
                    dFwaveInertia(1:3,cc) = dFwaveInertia(1:3,cc) + dFwI;
                    dFwaveInertia(4:6,cc) = dFwaveInertia(4:6,cc) + (-S*dFwI);
                end
            end % Type of element
            % Wave drag force and moment on part
            dFwaveDrag(1:3,cc) = dFwaveDrag(1:3,cc) + dFwD;
            dFwaveDrag(4:6,cc) = dFwaveDrag(4:6,cc) + (-S*dFwD);
            % Morison damping matrix for part
            dB_mor(1:3,:,cc) = dB_mor(1:3,:,cc) + dB;
            dB_mor(4:6,:,cc) = dB_mor(4:6,:,cc) + (-S*dB);
        end % All wave frequencies
        if L~=0 && first % Not heave plate and first iteration
            % Morison added mass matrix, related to inertia force
            dM = C_a*rho*h*pi*D^2/4 * (P*[eye(3) S]); 
            dA_mor(1:3,:) = dA_mor(1:3,:) + dM;
            dA_mor(4:6,:) = dA_mor(4:6,:) + (-S*dM);
        end
    end % Parts
    % Numerical integration along element  
    FwaveI(:,:,el) = dFwaveInertia;
    FwaveD(:,:,el) = dFwaveDrag;
    B_mor(:,:,:,el) = dB_mor;
    A_mor(:,:,el) = dA_mor;
end % Morison elements

% Total wave inertia and drag force
FtotwaveI(:,:) = sum(FwaveI,3);
FtotwaveD(:,:) = sum(FwaveD,3);
% Total added mass and damping matrices
Btot_mor(:,:,:) = sum(B_mor,4);
Atot_mor(:,:) = sum(A_mor,3);
% Heave plate 1 force and damping for internal load calculations
F_hp1 = FwaveD(:,:,4);
B_hp1 = B_mor(:,:,:,4);
end % Function 
